@@ -1,16 +1,16 @@
 ####################################################
 ### Authors: Simone Padoan and Moreno Bevilacqua.
-### Emails: simone.padoan@stat.unipd.it,
-### moreno.bevilacqua@unibg.it
-### Institutions: Department of Statistical Science,
-### University of Padua and Department of Information
-### Technology and Mathematical Methods, University
-### of Bergamo.
+### Emails: simone.padoan@unibocconi.it,
+### moreno.bevilacqua@uv.cl
+### Institutions: Department of Decision Sciences,
+### University Bocconi of Milan and
+### Departamento de Estadistica
+### Universidad de Valparaiso
 ### File name: Utility.r
 ### Description:
 ### This file contains a set of procedures
 ### for the set up of all the package routines.
-### Last change: 08/03/2012.
+### Last change: 28/03/2013.
 ####################################################
 
 ### Procedures are in alphabetical order.
@@ -33,19 +33,61 @@ CheckCorrModel <- function(corrmodel)
                              iacocesare=22,
                              porcu=23,
                              stein=24,
-                             # spatial-temporal non-separable correlations
+                             porcu2=25,
+                             gneiting_GC=26,
+                             porcu1=30,
+                             # spatial-temporal separable correlations
                              exp_cauchy=41,
                              exp_exp=42,
                              exp_gauss=43,
                              gauss_gauss=44,##not implemented
                              matern_cauchy=45,
-                             matern_exp=46)
+                             matern_exp=46,
+                             stab_stab=47,
+                             #Taper functions
+                             Wendland1=15,
+                             Wendland2=16,
+                             Wendland3=17,
+                             Wendland1_Wendland1=100,
+                             Wendland1_Wendland2=101,
+                             Wendland1_Wendland3=102,
+                             Wendland2_Wendland1=103,
+                             Wendland2_Wendland2=104,
+                             Wendland2_Wendland3=105,
+                             Wendland3_Wendland1=106,
+                             Wendland3_Wendland2=107,
+                             Wendland3_Wendland3=108,
+                             qt_time=109,
+                             qt_space=110)
     return(CheckCorrModel)
   }
 
-CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, grid, likelihood,
-                       lonlat, margins, maxdist, maxtime, model, numblock, optimizer, param,
-                       replicates, start, taper, threshold, type, varest, vartype, weighted)
+#CheckCorrTap <- function(corrtap)
+#  {
+#    CheckCorrTap <- NULL
+#    # Correlation function are in alphabetical order
+#    CheckCorrTap <- switch(corrtap,
+#                           Wendland1=15,
+#                           Wendland2=16,
+#                           Wendland3=17,
+#                           Wendland1_Wendland1=100,
+#                           Wendland1_Wendland2=101,
+#                           Wendland1_Wendland3=102,
+#                           Wendland2_Wendland1=103,
+#                           Wendland2_Wendland2=104,
+#                           Wendland2_Wendland3=105,
+#                           Wendland3_Wendland1=106,
+#                           Wendland3_Wendland2=107,
+#                           Wendland3_Wendland3=108,
+#                           qt_time=109,
+#                           qt_space=110)
+#    return(CheckCorrTap)
+#  }
+
+
+CheckInput <- function(coordx, coordy, coordt, corrmodel, data, distance, fcall, fixed, grid,
+                      likelihood, margins, maxdist, maxtime, model, numblock, optimizer, param,
+                       replicates, start, taper, tapsep, threshold, type, varest, vartype, weighted)
   {
     error <- NULL
     # START Include internal functions:
@@ -76,7 +118,23 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
         else CheckSpaceTime <- TRUE
         return(CheckSpaceTime)
     }
+    # Check the type of distances
+    CheckDistance<- function(distance)
+    {
+        CheckDistance <- NULL
+        CheckDistance <- switch(distance,
+                                eucl=0,
+                                Eucl=0,
+                                chor=1,
+                                Chor=1,
+                                geod=2,
+                                Geod=2,
+                                proj=3,
+                                Proj=3)
+        return(CheckDistance)
+    }
     # END Include internal functions
+    if(fcall!="Kriging"){
     ### START Checks inserted input
     # START common check fitting and simulation
     if(missing(coordx) || !is.numeric(coordx)){
@@ -103,15 +161,14 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
         error <- 'the model name of the random field is not correct\n'
         return(list(error=error))}
 
-#    if(model=="BinaryGauss"){
-#        error <- 'insert a numeric value for the threshold'
-#        if(is.null(threshold)) return(list(error=error))
-#        else if(!is.numeric(threshold)) return(list(error=error))}
-
     if(is.null(replicates) || (abs(replicates-round(replicates))>0) || replicates<1){
         error <- 'the parameter replicates need to be a positive integer\n'
         return(list(error=error))}
 
+    if(is.null(CheckDistance(distance))){
+        error <- 'the name of the distance is not correct\n'
+        return(list(error=error))}
+
     if(is.null(CheckCorrModel(corrmodel))){
         error <- 'the name of the correlation model is not correct\n'
         return(list(error=error))}
@@ -164,6 +221,16 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
                 error <- 'insert the correct type of marginal distributions\n'
                 return(list(error=error))}
 
+        if(model=="BrowResn")
+            if(CheckSpaceTime(CheckCorrModel(corrmodel))){
+                if(!corrmodel %in% c("exp_exp","exp_gauss","stable_stable")){
+                    error <- "the correlation model is not adequate for the Brown-Resnick process\n"
+                    return(list(error=error))}}
+            else
+                if(!corrmodel %in% c("exponential","gauss","gencauchy","stable")){
+                    error <- "the correlation model is not adequate for the Brown-Resnick process\n"
+                    return(list(error=error))}
+
         if(!is.null(optimizer) & !is.character(optimizer)){
             error <- 'insert the type of maximising algorithm\n'
             return(list(error=error))}
@@ -176,7 +243,11 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
         if(is.null(taper) || is.null(maxdist)){
           error <- 'tapering need a taper correlation model and/or a compact support\n'
           return(list(error=error))}
-        if(!taper %in% c("Wendland1","Wendland2","Wendland3")){
+        if(!taper %in% c("Wendland1","Wendland2","Wendland3",
+                         "Wendland1_Wendland1","Wendland1_Wendland2","Wendland1_Wendland3",
+                         "Wendland2_Wendland1","Wendland2_Wendland2","Wendland2_Wendland3",
+                         "Wendland3_Wendland1","Wendland3_Wendland2","Wendland3_Wendland3",
+                         "qt_time","qt_space")){
             error <- 'insert a correct name for the taper correlation model\n'
             return(list(error=error))}}
 
@@ -210,6 +281,11 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
             error <- 'the name of the estimation type for the variances is not correct\n'
             return(list(error=error))}
 
+          if(!is.null(tapsep)){
+            error <- "insert a numeric value between 0 and 1 for the separability parameter of the space time taper\n"
+            if(!is.numeric(tapsep)) return(list(error=error))
+            else if(tapsep<0||tapsep>1) return(list(error=error))}
+
         if(!is.null(start)){
             if(!is.list(start)){
                 error <- 'insert starting values as a list of parameters\n'
@@ -225,6 +301,10 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
                 error <- 'the mean parameter is not allow with the difference composite likelihood\n'
                 return(list(error=error))}
 
+            if(corrmodel=="gencauchy" && param["power1"]!=2){
+                error <- "the parameter power1 need to be equal to 2 for the Brown-Resnick process\n"
+                 return(list(error=error))}
+
             if(any(namstart=='sill') & (model=='BrowResn')){
                 error <- 'the sill parameter is not allow with Brown-Renick model\n'
                 return(list(error=error))}
@@ -238,10 +318,6 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
                     error <- 'some fixed parameter name/s is/are matching with starting parameter name/s\n'
                     return(list(error=error))}}
 
-        if(!is.null(lonlat) & !is.logical(lonlat)){
-            error <- 'insert the type of coordinates'
-            return(list(error=error))}
-
         if(!is.null(weighted) & !is.logical(weighted)){
             error <- 'insert if the composite likelihood need to be weighted'
             return(list(error=error))}
@@ -537,6 +613,31 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
     # END check fitting
     # START check simulation
     if(fcall=="Simulation"){
+        if(type=="Tapering"){
+        #if(is.null(taper) || is.null(maxdist) || is.null(maxtime)) {
+        #  error <- 'tapering need a taper correlation model and/or a compact support\n'
+        #  return(list(error=error))}
+        if(!taper %in% c("Wendland1","Wendland2","Wendland3",
+                         "Wendland1_Wendland1","Wendland1_Wendland2","Wendland1_Wendland3",
+                         "Wendland2_Wendland1","Wendland2_Wendland2","Wendland2_Wendland3",
+                         "Wendland3_Wendland1","Wendland3_Wendland2","Wendland3_Wendland3",
+                         "qt_time","qt_space")){
+            error <- 'insert a correct name for the taper correlation model\n'
+            return(list(error=error))}
+            if(!is.null(maxdist)){
+            error <- "insert a positive numeric value for the maximum spatial distance\n"
+            if(!is.numeric(maxdist)) return(list(error=error))
+            else if(maxdist<0) return(list(error=error))}
+            if(!is.null(maxtime)){
+            error <- "insert a positive numeric value for the maximum temporal distance\n"
+            if(!is.numeric(maxtime)) return(list(error=error))
+            else if(maxtime<0) return(list(error=error))}
+            if(!is.null(tapsep)){
+            error <- "separability parameter of spacetime taper must be between 0  and 1\n"
+            if(!is.numeric(tapsep)) return(list(error=error))
+            else if(tapsep<0||tapsep>1) return(list(error=error))}
+            }
+
         if(is.null(param) || !is.list(param)){
             error <- 'insert the parameters as a list\n'
             return(list(error=error))}
@@ -558,16 +659,48 @@ CheckInput <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gr
             if(!is.numeric(numblock)) return(list(error=error))
             else if(numblock<0) return(list(error=error))}
 
-        if(model=="BrowRen"){
-            if(!corrmodel %in% c("exponential","gauss","gencauchy","stable")){
+        if(model=="BrowResn"){
+            if(!corrmodel %in% c("exponential","gauss","gencauchy","stable","stable_stable")){
                 error <- "the correlation model is not adequate for the Brown-Resnick process\n"
                  return(list(error=error))}
             if(corrmodel=="gencauchy" && param["power1"]!=2){
                 error <- "the parameter power1 need to be equal to 2 for the Brown-Resnick process\n"
                  return(list(error=error))}}
+
     }
     # END check simulation
   }
+ else{    ### checking input for kriging
+   if(class(corrmodel)!="CovMat")   {
+           error <-  "covmatrix is not a object of class covmatrix\n"
+           return(list(error=error))}
+
+    if(missing(coordx)) {
+    error <- "spatial locations must be a matrix of dimension 2\n"
+    return(list(error=error))}
+    else     {
+    if(is.vector(coordx)&&!length(coordx)==2)    {
+           error <- "spatial locations must be a vector of dimension 2\n"
+           return(list(error=error))}
+    if(is.matrix(coordx)&&!ncol(coordx)==2)       {
+           error <- "spatial locations must be  a matrix of dimension 2\n"
+           return(list(error=error))}
+    }
+   if((!is.null(coordt)&&!is.numeric(coordt))  ){
+           error <- "time  must be a vector\n"
+           return(list(error=error))}
+   if(!type %in% c("simple","ordinary","Simple","Ordinary")){
+           error <-"kriging type can be  simple or Ordinary\n"
+   return(list(error=error))}
+   if(missing(data) || !is.numeric(data)){
+           error <- "insert a numeric vector of data\n"
+           return(list(error=error))}
+   if(length(data)!=nrow(corrmodel$covmatrix)) {
+           error <-  "data dimension does not correspond to the covmatrix dimension\n"
+           return(list(error=error))}
+        }
+      # END check kriging
+  }
 
 CheckLikelihood <- function(likelihood)
   {
@@ -636,18 +769,14 @@ CorrelationParam <- function(corrmodel)
       param <- c('scale', 'smooth')
       return(param)}
     # Non-separable spatial-temporal correlations:
-    # Gneiting model:
-    if(corrmodel=='gneiting'){
+    # Gneiting or Porcu model:
+    if(corrmodel=='gneiting'||corrmodel=='porcu'||corrmodel=='porcu1'||corrmodel=='porcu2'||corrmodel=='gneiting_GC'){
       param <- c('power_s', 'power_t','scale_s','scale_t','sep')
       return(param)}
     # Iaco-Cesare model:
     if(corrmodel=='iacocesare'){
       param <- c('power2','power_s', 'power_t','scale_s','scale_t')
       return(param)}
-    # Porcu model:
-    if(corrmodel=='porcu'){
-      param <- c('power_s', 'power_t','scale_s','scale_t','sep')
-      return(param)}
     # Stein model:
     if(corrmodel=='stein'){
       namesparam <- c('power_t','scale_s','scale_t','smooth_s')
@@ -669,6 +798,10 @@ CorrelationParam <- function(corrmodel)
      if(corrmodel=='matern_cauchy'){
        param <- c('power2','scale_s','scale_t','smooth_s')
        return(param)}
+    # Stable-Stable model:
+     if(corrmodel=='stab_stab'){
+       param <- c('power_s','power_t','scale_s','scale_t')
+       return(param)}
 
     return(param)
   }
@@ -695,9 +828,10 @@ NuisanceParam <- function(model)
   return(param)
 }
 
-InitParam <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, grid,
-                      likelihood, lonlat, margins, maxdist, maxtime, model, numblock,
-                      param, parscale, paramrange, replicates, start, threshold, type,
+
+InitParam <- function(coordx, coordy, coordt, corrmodel, data, distance, fcall, fixed, grid,
+                      likelihood, margins, maxdist, maxtime, model, numblock, param, parscale,
+                      paramrange, replicates, start, taper, tapsep, threshold, type,
                       typereal, varest, vartype, weighted, winconst, winstp)
 {
     ### START Includes internal functions:
@@ -709,6 +843,21 @@ InitParam <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gri
         else CheckSpaceTime <- TRUE
         return(CheckSpaceTime)
     }
+    # Check the type of distances
+    CheckDistance<- function(distance)
+    {
+        CheckDistance <- NULL
+        CheckDistance <- switch(distance,
+                                eucl=0,
+                                Eucl=0,
+                                chor=1,
+                                Chor=1,
+                                geod=2,
+                                Geod=2,
+                                proj=3,
+                                Proj=3)
+        return(CheckDistance)
+    }
     # Determines the range of the parameters for a given correlation
     SetRangeParam <- function(namesparam, numparam)
     {
@@ -788,6 +937,10 @@ InitParam <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gri
     else numcoord <- numcoordx <- numcoordy <- length(coordx)
     # initialize tapering variables:
     tapering<-ia<-idx<-ja<-integer(1)
+    nozero<-NULL
+    tapmodel=NULL
+    cutoff <- FALSE
+    distance<-CheckDistance(distance)
     ### END settings the data structure
     # START code for the simulation procedure
     if(fcall=="Fitting"){
@@ -797,12 +950,12 @@ InitParam <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gri
         vartype <- CheckVarType(vartype)
         type <- CheckType(type)
         if(model==1){ # Gaussian random field:
-            mu <- mean(data)
-            if(any(type==c(1, 3, 6)))# Checks the type of likelihood
-                if(is.list(fixed)) fixed$mean <- mu# Fixs the mean
-                else fixed <- list(mean=mu)
-            nuisance <- c(mu, 0, var(c(data)))
-            if(likelihood==2 & CheckType(typereal)==5) tapering <- 1}
+           mu <- mean(data)
+           if(any(type==c(1, 3, 6)))# Checks the type of likelihood
+           if(is.list(fixed)) fixed$mean <- mu# Fixs the mean
+           else fixed <- list(mean=mu)
+           nuisance <- c(mu, 0, var(c(data)))
+           if(likelihood==2 && CheckType(typereal)==5) tapering <- 1}
         if(model==2){ # Binary Gaussian random field:
             # check the threshould:
             if(is.null(threshold) || !is.numeric(threshold))
@@ -876,16 +1029,26 @@ InitParam <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gri
                     dim(data) <- c(numcoord, numtime, replicates)
                     data <- aperm(data, c(2,1,3))}
                 else # if there are not iid replicates:
-                    data <- matrix(data, ncol=numcoord, nrow=numtime, byrow=TRUE)}
+                    data <- matrix(data, ncol=numcoord, nrow=numtime, byrow=TRUE)
+                if(typereal=="Tapering"){
+                tapering<-1
+                idx<-integer((numcoord*numtime)^2)
+                ja<-integer((numcoord*numtime)^2)
+                ia<-integer(numcoord*numtime+1)
+                tapmodel<-CheckCorrModel(taper)
+                }}
         else{
             numtime <- 1
             coordt <- 0
             if(grid) data <- matrix(data, ncol=numcoord, nrow=replicates, byrow=TRUE)
             else data <- matrix(data, ncol=numcoord, nrow=replicates)
             if(typereal=="Tapering"){
-                idx<-integer(numcoord*numcoord)
-                ja<-integer(numcoord*numcoord)
-                ia<-integer(numcoord+1)}}
+                tapering<-1
+                idx<-integer((numcoord*numtime)^2)
+                ja<-integer((numcoord*numtime)^2)
+                ia<-integer(numcoord*numtime+1)
+                tapmodel<-CheckCorrModel(taper)
+                }}
     }
     # END code for the fitting procedure
     # START code for the simulation procedure
@@ -905,29 +1068,42 @@ InitParam <- function(coordx, coordy, coordt, corrmodel, data, fcall, fixed, gri
         namessim <- c("mean","sill","nugget","scale",namescorr[!namescorr=="scale"])
         if(spacetime) numtime <- length(coordt)
         else {numtime <- 1; coordt <- 0}
-    }
+        if(typereal=="Tapering"&&type=="Tapering"){
+                tapering<-1
+                idx<-integer((numcoord*numtime)^2)
+                ja<-integer((numcoord*numtime)^2)
+                ia<-integer(numcoord*numtime+1)
+                tapmodel<-CheckCorrModel(taper)
+                }
+        if(model==2){ # Binary Gaussian random field:
+            # check the threshould:
+            if(is.null(threshold) || !is.numeric(threshold))
+                threshold <- 0 }
+        }
     # END code for the simulation procedure
     ### Compute the spatial and spatial-temporal distances:
     numpairs <- integer(1)
     srange <- double(1)
     trange <- double(1)
-    if(is.null(maxdist)) srange<-c(srange, double(1)) else srange<-c(srange, as.double(maxdist))
-    if(is.null(maxtime)) trange<-c(trange, double(1)) else trange<-c(trange, as.double(maxtime))
+    if(is.null(maxdist)) srange<-c(srange,double(1)) else {srange<-c(srange,as.double(maxdist))}                # cutoff<-TRUE
+    if(is.null(maxtime)) trange<-c(trange,double(1)) else {trange<-c(trange,as.double(maxtime))}                # cutoff<-TRUE
     isinit <- as.integer(1)
-    .C('SetGlobalVar',as.double(coordx),as.double(coordy),as.double(coordt),
-       as.integer(grid),ia,idx,isinit,ja,as.integer(numcoord),as.integer(numcoordx),
-       as.integer(numcoordy),numpairs,as.integer(replicates),as.integer(spacetime),
-       srange,as.integer(numtime),trange,as.integer(tapering),as.integer(lonlat),
-       as.integer(weighted), PACKAGE='CompRandFld', DUP=FALSE, NAOK=TRUE)
+    .C('SetGlobalVar',as.double(coordx),as.double(coordy),as.double(coordt),as.integer(grid),ia,idx,
+       isinit,ja,as.integer(numcoord),as.integer(numcoordx), as.integer(numcoordy),numpairs,as.integer(replicates),
+       srange, as.double(tapsep), as.integer(numtime),trange,as.integer(tapering),as.integer(tapmodel),
+       as.integer(distance),as.integer(weighted),PACKAGE='CompRandFld', DUP=FALSE, NAOK=TRUE)
+     if(tapering){ nozero=numpairs/(numcoord*numtime)^2
+                   idx <- idx[1:numpairs]
+                   ja <- ja[1:numpairs]}
     ### Returned list of objects:
-    return(list(coordx=coordx,coordy=coordy,coordt=coordt,corrmodel=corrmodel,data=data,
+    return(list(coordx=coordx,coordy=coordy,coordt=coordt,corrmodel=corrmodel,data=data,distance=distance,
                 error=error,flagcorr=flagcorr,flagnuis=flagnuis,fixed=fixed,likelihood=likelihood,
                 lower=paramrange$lower,model=model,namescorr=namescorr,namesfixed=namesfixed,
                 namesnuis=namesnuis,namesparam=namesparam,namessim=namessim,namesstart=namesstart,
                 numblock=numblock,numcoord=numcoord,numcoordx=numcoordx,numcoordy=numcoordy,
                 numfixed=numfixed,numpairs=numpairs,numparam=numparam,numparamcorr=numparamcorr,
-                numrep=replicates,numstart=numstart,numtime=numtime,param=param,setup=list(
-                ia=ia,idx=idx,ja=ja,nozero=numpairs/numcoord^2),
+                numrep=replicates,numstart=numstart,numtime=numtime,param=param,setup=list(                ## setup is a list
+                ia=ia,idx=idx,ja=ja,nozero=nozero,tapmodel=tapmodel,tapsep=tapsep),                              ## with tapered matrix informations
                 spacetime=spacetime,srange=srange,start=start,upper=paramrange$upper,type=type,
                 threshold=threshold,trange=trange,vartype=vartype,winconst=winconst,winstp=winstp))
 }
