@@ -8,8 +8,12 @@
 #define MAXERR 1e-6
 
 //---------START GLOBAL VARIABLES-----------
-double *lags;// vector of spatial distances
-double *lagt;// vector of temporal distances
+int *isst;//is a spatio-temporal random field?
+int *istap;//is tapering?
+double *lags;// vector of spatial distances for tapering
+double *lagt;// vector of temporal distance for tapering
+double **mlags;// vector of spatial distances
+double **mlagt;// vector of temporal distances
 double *maxdist;// the threshould of the spatial distances
 double *maxtime;// the threshould of the temporal distances
 // below which the pairs are considered
@@ -17,15 +21,14 @@ double *maximdista;// the maximum spatial distance
 double *maximtime;// the maximum temporal distance
 double *minimdista; // the minimum spatial distance
 double *minimtime;// the minimum temporal distance
-double **mlags;// matrix of spatial distances
-double **mlagt;// matrix of temporal distances
 int *ncoord;// number of total spatial coordinates
 int *ncoordx;// number of the first spatial coordinates
 int *ncoordy;// number of the second spatial coordinates
-int *npairs;// numner of spatial pairs
-int *npairt;// number of temporal times
+int *npairs;// effective number of pairs
 int *nrep;// number of iid replicates of the random field
 int *ntime;// number of times
+double *tapsep; // parameter separability for space time quasi taper
+//int *totpairs;// total number of pairs
 //---------END GLOBAL VARIABLES-------------
 //void indx(double *ind, int *n);
 
@@ -42,6 +45,8 @@ Start
 
 double CheckCor(int *cormod, double *par);
 
+void Comp_supp(double *c_supp,int *cormod, double h,double u, double *par);
+
 double CorFct(int *cormod, double h, double u, double *par);
 
 double CorFunCauchy(double lag, double power2, double scale);
@@ -52,21 +57,36 @@ double CorFunSferical(double lag, double scale);
 
 double CorFunStable(double lag, double power, double scale);
 
+
+		       //double CorFunDobStable(double lag, double power_s, double power_t,
+		       //double scale_s, double scale_t, double tsep);
+
 double CorFunWitMat(double lag, double scale, double smooth);
 
-double CorFunWend1(double lag);
+double CorFunWend1(double lag,double scale);
 
-double CorFunWend2(double lag);
+double CorFunWend2(double lag,double scale);
 
-double CorFunWend3(double lag);
+double CorFunWend3(double lag,double scale);
 
 void CorrelationMat(double *rho, int *cormod, double *nuis, double *par);
 
+void CorrelationMat_tap(double *rho, int *cormod, double *nuis, double *par);
+
 void CorrelationMat_st(double *rho, int *cormod, double *nuis, double *par);
 
+void Corr_c(double *cc,double *coordx, double *coordy, double *coordt, int *cormod, int *grid, double *locx,  double *locy,int *ncoord, int *nloc,int*tloc,
+                 int *ntime, double *par, int *spt, double *time,int *type);
+
+void Corr_c_tap(double *cc,double *cc_tap,double *coordx, double *coordy, double *coordt, int *cormod, int *cormodtap, int *grid, double *locx,  double *locy,int *ncoord, int *nloc,int*tloc,
+                 int *ntime, double *par, int *spt, double *time,int *type);
+
 void DCorrelationMat(int *cormod,double *drho,double *eps,int *flagcor,
 		     int *nparcor, double *parcor,double *rho);
 
+void DCorrelationMat_tap(int *cormod,double *drho,double *eps,int *flagcor,
+		     int *nparcor, double *parcor,double *rho);
+
 void DCorrelationMat_st(int *cormod,double *drho,double *eps,int *flagcor,
 			int *nparcor, double *parcor,double *rho);
 
@@ -165,15 +185,15 @@ void GradCorrFct(double rho, int *cormod, double *eps, int *flag,
 		 double *grad, double h, double u, double *par);
 
 void GradVarioFct(double vario, int *cormod, double *eps, int *flag,
-		  double *grad, double lag, double *par);
+		  double *grad, double lag, double *par, double tsep);
 
 void TapVectCorrelation(double *rho,int *cormod,double *tdists,int *ntdists,double *nuis,double *par);
 
 double Variogram(int *cormod, double h, double u, double *nuis, double *par);
 
-double VarioFct(int *cormod, double lag, double *par);
+double VarioFct(int *cormod, double h, double *par, double u);
 
-double VarioStable(double lag, double power, double scale);
+double VarioDobStable(double lag, double power_s, double power_t, double scale_s, double scale_t, double tsep);
 
 double VarioGCauchy(double lag, double power1, double power2, double scale);
 
@@ -193,6 +213,8 @@ Description: functions for composite log-likelihood evaluation
 Start
  ---------------------------------------------------------------*/
 
+double BrowResnllik(double a, double c, double x, double y);
+
 void Comp_Cond_Gauss(int *cormod, double *data, double *nuis, double *par, double *thr, double *res);
 
 void Comp_Cond_Gauss_st(int *cormod, double *data, double *nuis, double *par, double *thr, double *res);
@@ -219,10 +241,13 @@ void Comp_Pair_BinGauss_st(int *cormod, double *data, double *nuis, double *par,
 
 void Comp_Brow_Resn(int *cormod, double *data, double *nuis, double *par, double *thr, double *res);
 
+void Comp_Brow_Resn_st(int *cormod, double *data, double *nuis, double *par, double *thr, double *res);
+
 void Comp_Ext_Gauss(int *cormod, double *data, double *nuis, double *par, double *thr, double *res);
 
 void Comp_Ext_T(int *cormod, double *data, double *nuis, double *par, double *thr, double *res);
 
+
 /*----------------------------------------------------------------
 File name: CompositeLikelihood.c
 Description: functions for composite log-likelihood evaluation
@@ -284,13 +309,52 @@ Description: procedures for the computation of the Godambe matrix
 Start
  ---------------------------------------------------------------*/
 
-void GodambeMat_emp(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
-		    int *like, int *model, int *npar, int *nparc, double *parcor,
-		    double *nuis, double *score, double *sensmat, double *varimat,
-		    int *type);
+void God_Cond_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		    int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		    double *sensmat, double *varimat);
 
-void GodambeMat(double *coordx, double *coordy, int *cormod, double *data, double *eps,
-		int *flagcor, int *flagnuis, int *grid, int *like, int *lonlat, int *model,
+void God_Cond_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		       int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		       double *sensmat, double *varimat);
+
+void God_Diff_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		    int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		    double *sensmat, double *varimat);
+
+void God_Diff_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		       int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		       double *sensmat, double *varimat);
+
+void God_Pair_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		    int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		    double *sensmat, double *varimat);
+
+void God_Pair_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		       int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		       double *sensmat, double *varimat);
+
+void God_BrowResn(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		   int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		   double *sensmat, double *varimat);
+
+void God_BrowResn_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		     int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		     double *sensmat, double *varimat);
+
+void God_Ext_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		   int *npar, int *nparc, double *parcor, double *nuis, double *score,
+		   double *sensmat, double *varimat);
+
+void God_Ext_T(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+	       int *npar, int *nparc, double *parcor, double *nuis, double *score,
+	       double *sensmat, double *varimat);
+
+void GodambeMat_emp(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
+		    int *like, int *model, int *npar, int *nparc, double *parcor, double *nuis,
+		    double *score, double *sensmat, int *spt, double *varimat, int *type);
+
+void GodambeMat(double *coordx, double *coordy, int *cormod, double *data, int *dist,
+		double *eps,int *flagcor, int *flagnuis, int *grid, int *like, int *model,
 		int *npar, int *nparc, double *parcor, double *nuis, double *score,
 		double *sensmat, int *spt, double *thr, int *type, double *varimat,
 		int *vartype, double *winc, double *winstp);
@@ -299,48 +363,45 @@ void GodambeMat_Diff(double *coordx, double *coordy, int *cormod, double *eps, i
 		     int *flagnuis, int *model, int *npar, int *nparc, double *parcor,
 		     double *nuis, double *sensmat, double *varimat);
 
-void Sens_Cond_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis, double *nuis,
-		     int *npair, int *npar, int *nparc, double *parcor, double *score, double *sensmat);
+void Sens_Cond_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis, double *nuis, int *np,
+		     int *npar, int *nparc, double *parcor, double *score, double *sensmat);
 
-void Sens_Cond_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis, double *nuis,
-			int *npair, int *npar, int *nparc, double *parcor, double *score, double *sensmat);
+void Sens_Cond_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis, double *nuis, int *np,
+			int *npar, int *nparc, double *parcor, double *score, double *sensmat);
 
 void Sens_Cond_Gauss_ij(double rho, int *flag, double *gradcor, int *npar,
 			int *nparc, double *par, double *sensmat);
 
 void Sens_Diff_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
-		     double *nuis, int *npair, int *npar, int *nparc, double *parcor,
-		     double *score, double *sensmat);
+		     double *nuis, int *np,int *npar, int *nparc, double *parcor,double *score, double *sensmat);
 
 void Sens_Diff_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
-			double *nuis, int *npair, int *npar, int *nparc, double *parcor,
-			double *score, double *sensmat);
+			double *nuis, int *np,int *npar, int *nparc, double *parcor, double *score, double *sensmat);
 
 void Sens_Diff_Gauss_ij(double *gradient, int *npar, double *sensmat);
 
 void Sens_Pair_Gauss(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
-		     double *nuis, int *npair, int *npar, int *nparc, double *parcor,
-		     double *score, double *sensmat);
+		     double *nuis, int *np, int *npar, int *nparc, double *parcor, double *score, double *sensmat);
 
 void Sens_Pair_Gauss_st(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis, double *nuis,
-			int *npair, int *npar, int *nparc, double *parcor, double *score, double *sensmat);
+			int *np, int *npar, int *nparc, double *parcor, double *score, double *sensmat);
 
 void Sens_Pair_Gauss_ij(double rho, int *flag, double *gradcor, int *npar,
 			int *nparc, double *par, double *sensmat);
 
 void Sensitivity(int *cormod, double *data, double *eps, int *flagcor, int *flagnuis,
-		 int *like, int *model, int *npair, int *npar, int *nparc, double *parcor,
-		 double *nuis, double *score, double *sensmat, int *spt, int *type);
+		 int *like, int *model, int *npar, int *nparc, double *parcor,
+		 double *nuis, int *np, double *score, double *sensmat, int *spt, int *type);
 
 void Vari_SubSamp(double *coordx, double *coordy, int *cormod, double *data,
-		  double *eps, int *flagcor, int *flagnuis, int *grid, int *like,
-		  int *lonlat, int *model, int *npair, int *npar, int *nparc,
-		  double *nuis, double *parcor, double *thr, int *type, double *varimat,
+		  int *dist, double *eps, int *flagcor, int *flagnuis, int *grid,
+            int *like,int *model, int *npar, int *nparc, double *nuis, int *np,
+		  double *parcor, double *thr, int *type, double *varimat,
 		  double *winc, double *winstp);
 
-void Vari_SubSamp_st(int *cormod, double *data, double *eps, int *flagcor,
-		     int *flagnuis, int *like, int *npair, int *npar, int *nparc,
-		     double *nuis, double *parcor, int *type, double *varimat, double *winc, double *winstp);
+void Vari_SubSamp_st(int *cormod, double *data, int *dist, double *eps, int *flagcor,int *flagnuis,
+		     int *like, int *npar, int *nparc, double *nuis, int *np,double *parcor,
+		     int *type, double *varimat, double *winc, double *winstp);
 
 /*----------------------------------------------------------------
 File name: Godambe.c
@@ -426,8 +487,14 @@ void Binned_Lorelogram_st(double *bins, double *bint, double *data, int *lbins,
 
 void Binned_Madogram(double *bins, double *data, int *lbins, double *moments, int *nbins);
 
+void Binned_Madogram_st(double *bins, double *bint, double *data, int *lbins,
+			int *lbinst, int *lbint, double *moms,double *momst,
+			double *momt, int *nbins, int *nbint);
+
 void Binned_Variogram(double *bins, double *data, int *lbins, double *moms, int *nbins);
 
+void Binned_Variogram_2(double *bins, double *data, int *lbins, double *moms, int *nbins);
+
 void Binned_Variogram_st(double *bins, double *bint, double *data, int *lbins,
 			 int *lbinst, int *lbint, double *moms, double *momst,
 			 double *momt, int *nbins, int *nbint);
@@ -472,9 +539,11 @@ void DeleteGlobalVar();
 
 void RangeDist(double *max, double *min);
 
-double Dist_geodesic(double lonx, double latx, double lony, double laty);
+double Dist_geodesic(double loni, double lati, double lonj, double latj);
 
-void GeoDist(double *coordx, double *coordy, int *ncoord, double *res);
+double Dist_chordal(double loni, double lati, double lonj, double latj);
+
+void GeoDist(double *coordx, double *coordy, int *ncoord, double *res,int *type_dist);
 
 void ComputeMaxima(double *df, double *maxima, int *model,
 		   int *nblock, int *nsite, double *sim);
@@ -500,13 +569,14 @@ void SetSampling_st(double *data,double *sdata,int *ncoord,int *ntime,
 
 void SetGlobalVar(double *coordx,double *coordy,double *coordt,int *grid,int *ia,
 		  int *idx,int *ismal,int *ja,int *nsite,int *nsitex,int *nsitey,
-		  int *npair,int *replic,int *spatim,double *srange,int *times,
-		  double *trange,int *tap,int *type,int *weighted);
+		  int *npair,int *replic,double *srange, double *sep, int *times,double *trange,
+		  int *tap,int *tapmodel,int *type,int *weighted);
 
 void Space_Dist(double *coordx,double *coordy,int grid,int *ia,int *idx,
-		int *ja,double tapran,int tap,int type);
+		int *ismal,int *ja,double thres,int type);
 
-void SpaceTime_Dist(double *coordx, double *coordy, double *coordt, int *grid, int *type);
+void SpaceTime_Dist(double *coordx,double *coordy,double *coordt,int *grid,int *ia,int *idx,int *ismal,int *ja,
+                    int *tapmodel,double thres,double thret,int type);
 
 /*----------------------------------------------------------------
 File name: Utility.c
